/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../texture.h"

/** Which character (ASCII) to start the font from */
#define FONT_FIRST_CHAR 32

/** How many characters (after the first char) to generate */
#define FONT_NUM_CHARS 96

/** Width of the loaded font texture */
#define FONT_TEXTURE_WIDTH 1024

/** Height of the loaded font texture */
#define FONT_TEXTURE_HEIGHT FONT_TEXTURE_WIDTH

/** Refer to STBTT docs for OpenGL Fill Mode v d3d Fill Modes */
#define FONT_FILL_MODE 1

/** Passed to STBTT for scaling the font, essentially the font resolution */
#define FONT_TEXTURE_SIZE 64

/** Default Font Size (on which all font scales are based) */
#define FONT_SIZE_DEFAULT (FONT_TEXTURE_SIZE / 2)

// Chars
#define FONT_NEWLINE '\n'
#define FONT_SPACE ' '

// Heights
#define FONT_LINE_HEIGHT FONT_TEXTURE_SIZE*0.75
#define FONT_INITIAL_LINE FONT_LINE_HEIGHT*0.75
#define FONT_SPACE_SIZE FONT_TEXTURE_SIZE*0.45

/** Maximum number of characters a font text info can hold. */
#define FONT_TEXT_INFO_CHARS_MAX 1024

/** Maximum number of newlines that a font text info can hold. */
#define FONT_TEXT_INFO_LINES_MAX FONT_TEXT_INFO_CHARS_MAX/50

/** Representation of a font that can be used to render text */
typedef struct {
  texture_t texture;
  stbtt_bakedchar characterData[FONT_NUM_CHARS];
} font_t;

typedef struct {
  /** What (real character) index the line starts at */
  int32_t start;
  /** How many (real) characters the line is in length */
  int32_t length;
} fonttextinfoline_t;

typedef struct {
  /** How many raw chars are in the string */
  int32_t length;

  /** How many real characters (non whitespace) are in the string */
  int32_t realLength;

  /** How many lines is the string? Trailing newlines will count */
  int32_t lineCount;

  /** The real character info for each line */
  fonttextinfoline_t lines[FONT_TEXT_INFO_LINES_MAX];

  /** Dimensions of the string */
  float width, height;

  /** Array of precalculated quads */
  stbtt_aligned_quad quads[FONT_TEXT_INFO_CHARS_MAX];
} fonttextinfo_t;